package gol;

import org.junit.jupiter.api.Test;
import static org.junit.jupiter.api.Assertions.*;
import static gol.GameOfLife.CellState.*;
import static gol.GameOfLife.*;
import java.awt.Point;
import java.util.*;

/*
Kent Beck's 4 rules of simple design
-Passes all the tests
-Reveals intention
-No duplication
-Fewest elements
*/

class GameOfLifeTest {
  @Test
  public void canary() {
    assertTrue(true);
  }
  
  @Test
  void deadCellBehavior() {
    assertAll(
      () -> assertEquals(DEAD, nextState(DEAD, 0)),
      () -> assertEquals(DEAD, nextState(DEAD, 1)),
      () -> assertEquals(DEAD, nextState(DEAD, 2)),
      () -> assertEquals(DEAD, nextState(DEAD, 5)),
      () -> assertEquals(DEAD, nextState(DEAD, 8)),
      () -> assertEquals(ALIVE, nextState(DEAD, 3))
    );
  }
  
  @Test
  void liveCellBehavior() {
    assertAll(
      () -> assertEquals(DEAD, nextState(ALIVE, 0)),
      () -> assertEquals(DEAD, nextState(ALIVE, 1)),
      () -> assertEquals(DEAD, nextState(ALIVE, 5)),
      () -> assertEquals(DEAD, nextState(ALIVE, 8)),      
      () -> assertEquals(ALIVE, nextState(ALIVE, 2)),
      () -> assertEquals(ALIVE, nextState(ALIVE, 3))      
    );
  }
  
  @Test
  void generateSignalsForPosition32() {
    assertEquals(List.of(
      new Point(2, 1), new Point(2, 2), new Point(2, 3), 
      new Point(3, 1), new Point(3, 3), 
      new Point(4, 1), new Point(4, 2), new Point(4, 3)),
      generateSignalsForAPosition(new Point(3, 2)));
  }

  @Test
  void generateSignalsForPosition00() {
    assertEquals(List.of(
      new Point(-1, -1), new Point(-1, 0), new Point(-1, 1), 
      new Point(0, -1), new Point(0, 1), 
      new Point(1, -1), new Point(1, 0), new Point(1, 1)),
      generateSignalsForAPosition(new Point(0, 0)));
  }

  @Test
  void generateSignalsForListOfPositionsWithOnePosition() {
    assertEquals(List.of(
      new Point(2, 1), new Point(2, 2), new Point(2, 3), 
      new Point(3, 1), new Point(3, 3), 
      new Point(4, 1), new Point(4, 2), new Point(4, 3)),
      generateSignalsForPositions(List.of(new Point(3, 2))));
  }

  @Test
  void generateSignalsForListOfPositionsWithTwoPositions() {
    assertEquals(List.of(
      new Point(2, 1), new Point(2, 2), new Point(2, 3), 
      new Point(3, 1), new Point(3, 3), 
      new Point(4, 1), new Point(4, 2), new Point(4, 3),
      new Point(-1, -1), new Point(-1, 0), new Point(-1, 1), 
      new Point(0, -1), new Point(0, 1), 
      new Point(1, -1), new Point(1, 0), new Point(1, 1)),
      generateSignalsForPositions(List.of(new Point(3, 2), new Point(0, 0))));
  }
  
  @Test
  void countSignalsForZeroPositions() {
    assertEquals(Map.of(), countSignals(List.of()));
  }

  @Test
  void countSignalsForOnePositions() {
    assertEquals(Map.of(new Point(4, 3), 1L),
      countSignals(List.of(new Point(4, 3))));
  }

  @Test
  void countSignalsForTwoOfTheSamePosition() {
    assertEquals(Map.of(new Point(4, 3), 2L),
      countSignals(List.of(new Point(4, 3), new Point(4, 3))));
  }

  @Test
  void countSignalsForThreePositionsWithTwoOfSame() {
    assertEquals(Map.of(new Point(4, 3), 2L, new Point(3, 1), 1L),
      countSignals(
        List.of(new Point(4, 3), new Point(3, 1), new Point(4, 3))));
  }
  
  @Test
  void aDeadUniverseIsDead() {
    assertEquals(List.of(), nextGeneration(List.of()));
  }

  @Test
  void aUniverseWithLoneAliveCellIsDead() {
    assertEquals(List.of(), nextGeneration(List.of(new Point(4, 7))));
  }

  @Test
  void aUniverseWith7ShapeIsABlock() {
    List<Point> nextGen = nextGeneration(
      List.of(new Point(1, 2), new Point(1, 3), new Point(2, 3)));
  
      assertEquals(4, nextGen.size());
      assertTrue(nextGen.contains(new Point(1, 2)));
      assertTrue(nextGen.contains(new Point(1, 3)));
      assertTrue(nextGen.contains(new Point(2, 2)));
      assertTrue(nextGen.contains(new Point(2, 3)));
  }

  @Test
  void horizontalBlinkerBecomesVerticalBlinker() {
    List<Point> nextGen = nextGeneration(
      List.of(new Point(2, 2), new Point(2, 3), new Point(2, 4)));
  
      assertEquals(3, nextGen.size());
      assertTrue(nextGen.contains(new Point(1, 3)));
      assertTrue(nextGen.contains(new Point(2, 3)));
      assertTrue(nextGen.contains(new Point(3, 3)));
  }

  @Test
  void verticalBlinkerBecomesHorizontalBlinker() {
    List<Point> nextGen = nextGeneration(
      List.of(new Point(1, 3), new Point(2, 3), new Point(3, 3)));
  
      assertEquals(3, nextGen.size());
      assertTrue(nextGen.contains(new Point(2, 2)));
      assertTrue(nextGen.contains(new Point(2, 3)));
      assertTrue(nextGen.contains(new Point(2, 4)));
  }
  
  @Test
  void boundsForEmpty() {
    assertEquals(List.of(), boundsFor(List.of()));
  }

  @Test
  void boundsForOneValue() {
    assertEquals(List.of(new Point(4, 3), new Point(4, 3)),
      boundsFor(List.of(new Point(4, 3))));
  }

  @Test
  void boundsForTwoValue() {
    assertEquals(List.of(new Point(4, 3), new Point(7, 5)),
      boundsFor(List.of(new Point(7, 5), new Point(4, 3))));
  }
}





